<?php


/**
 * Set up the Bitcoin payment method settings form.
 */
function uc_bitcoin_settings_form() {
  $form = array();
  $path = drupal_get_path('module', 'uc_bitcoin');

  $form['server']['uc_bitcoin_conn_type'] = array(
    '#type' => 'select',
    '#options' => drupal_map_assoc(array('Bitcoin daemon', 'Blockchain.info')),
    '#title' => t('Select the backend type (server direct or blockchain.info)'),
    '#default_value' => variable_get('uc_bitcoin_conn_type', UC_BITCOIN_DEFAULT_SERVER_TYPE),
  );

  // BLOCKCHAIN.INFO Config

  $form['server']['connection']['blockchain_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Blockchain.info'),
    '#states' => array(
      'visible' => array(
        'select[name=uc_bitcoin_conn_type]' => array('value' => 'Blockchain.info'),
      ),
    ),
  );


  $form['server']['connection']['blockchain_info']['logo'] = array('#value' => theme('image', $path . '/images/blockchain-logo.png'));


  // Fields that we need
  // https://blockchain.info/merchant/$guid/new_address?password=$main_password&second_password=$second_password&label=$label
  $form['server']['connection']['blockchain_info']['uc_bitcoin_blockchain_guid'] = array(
    '#type' => 'textfield',
    '#title' => t('GUID (Also known as your Wallet ID)'),
    '#default_value' => variable_get('uc_bitcoin_blockchain_guid', ''),
    '#description' => t("For example") . ' ' . '00000000-0000-0000-0000-000000000000'
  );

  $form['server']['connection']['blockchain_info']['uc_bitcoin_blockchain_main_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Main password'),
    '#default_value' => variable_get('uc_bitcoin_blockchain_main_password', ''),
    '#description' => t('Main password')
  );

  $form['server']['connection']['blockchain_info']['uc_bitcoin_blockchain_second_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Second password'),
    '#default_value' => variable_get('uc_bitcoin_blockchain_second_password', ''),
    '#description' => t('Second password') . ' - ' . t('generally used for two factor authentication')
  );

  // BITCOIN direct to server configuration
  $form['server']['connection']['bitcoind'] = array(
    '#type' => 'fieldset',
    '#title' => t('Direct Bitcoin server connection'),
    '#states' => array(
      'visible' => array(
        'select[name=uc_bitcoin_conn_type]' => array('value' => UC_BITCOIN_DEFAULT_SERVER_TYPE),
      ),
    ),
  );

  $form['server']['connection']['bitcoind']['logo'] = array('#value' => theme('image', $path . '/images/WeLv_BC_48px.png'));

  $form['server']['connection']['bitcoind']['uc_bitcoin_server_scheme'] = array(
    '#type' => 'radios',
    '#title' => t('Server scheme'),
    '#default_value' => variable_get('uc_bitcoin_server_scheme', "http"),
    '#options' => array('http' => 'HTTP', 'https' => 'HTTPS'),
  );
  $form['server']['connection']['bitcoind']['uc_bitcoin_server_certificate_path'] = array(
    '#type' => 'textfield',
    '#title' => t('SSL certificate path'),
    '#default_value' => variable_get('uc_bitcoin_server_certificate_path', ''),
    '#description' => t('Location of the server.cert file you generated for bitcoind, if you are using HTTPS, more information !link',
      array('!link' => '<a href="https://en.bitcoin.it/wiki/Enabling_SSL_on_original_client_daemon">' . t('available here') . '</a>')),
  );
  $form['server']['connection']['bitcoind']['uc_bitcoin_server_username'] = array(
    '#type' => 'textfield',
    '#title' => t('Server username'),
    '#default_value' => variable_get('uc_bitcoin_server_username', ''),
    '#description' => t("User name for your Bitcoin server's JSON-RPC-HTTP interface"),
  );
  $form['server']['connection']['bitcoind']['uc_bitcoin_server_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Server password'),
    '#default_value' => variable_get('uc_bitcoin_server_password', ''),
    '#description' => t('Bitcoin server password'),
  );
  $form['server']['connection']['bitcoind']['uc_bitcoin_server_address'] = array(
    '#type' => 'textfield',
    '#title' => t('Server hostname'),
    '#default_value' => variable_get('uc_bitcoin_server_address', 'localhost'),
    '#description' => t('Bitcoin server domain name or IP address. Use "localhost" if bitcoind is running on the Ubercart server.'),
  );
  $form['server']['connection']['bitcoind']['uc_bitcoin_server_port'] = array(
    '#type' => 'textfield',
    '#title' => t('Server port'),
    '#default_value' => variable_get('uc_bitcoin_server_port', '8332'),
    '#description' => t('Bitcoin server port (generally 8332 or 443 if you have selected HTTPS above)'),
  );
  $form['transaction'] = array(
    '#type' => 'fieldset',
    '#title' => t('Transaction settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['transaction']['uc_bitcoin_rate_lookup'] = array(
    '#type' => 'select',
    '#title' => t('Preferred service for exchange rate lookup'),
    '#default_value' => variable_get('uc_bitcoin_rate_lookup', UC_BITCOIN_DEFAULT_RATE_LOOKUP),
    '#required' => TRUE,
    '#options' => array(
      'blockchain.info' => 'Blockchain.info',
      'coinbase' => 'Coinbase'
    )
  );

  $form['transaction']['uc_bitcoin_transaction_timeout'] = array(
    '#type' => 'textfield',
    '#title' => t('Transaction timeout (hours)'),
    '#default_value' => variable_get('uc_bitcoin_transaction_timeout', '72'),
    '#size' => 4,
    '#description' => t('Transactions not paid for within this number of hours will be automatically cancelled. Decimals are permitted.'),
    '#required' => TRUE,
  );


  $options = array();
  for ($i = 0; $i <= 10; $i++) {
    $options[$i] = $i;
  }
  $form['transaction']['uc_bitcoin_transaction_confirms'] = array(
    '#type' => 'select',
    '#title' => t('Minimum transaction confirmations'),
    '#default_value' => variable_get('uc_bitcoin_transaction_confirms', UC_BITCOIN_DEFAULT_MIN_TRANSACTIONS),
    '#description' => t('Minimum number of Bitcoin transaction network confirmations required before a payment is considered accepted.'),
    '#required' => TRUE,
    '#options' => $options
  );

  $form['#validate'][] = 'uc_bitcoin_settings_form_validate';

  return $form;
}

/**
 * Validate Bitcoin payment method settings.
 * @param unknown_type $form
 * @param unknown_type $form_state
 */
function uc_payment_methods_form_validate($form, &$form_state) {

  if($form_state['values']['uc_bitcoin_conn_type'] == 'Bitcoin daemon') {
    $c = $form_state['values']['uc_bitcoin_server_scheme'];
    if ($c != 'http' && $c != 'https') {
      form_set_error('uc_bitcoin_server_scheme', t('Server scheme must be HTTP or HTTPS.'));
    }

    if ($c == 'https') {
      $c = $form_state['values']['uc_bitcoin_server_certificate_path'];
      if (!empty($c) && !is_readable($c)) {
        form_set_error('uc_bitcoin_server_certificate_path', t('SSL certificate file unreadable. Try using an absolute path or check file permissions.'));
      }
    }

    $c = $form_state['values']['uc_bitcoin_server_address'];
    if (!drupal_valid_http_host($c)) {
      form_set_error('uc_bitcoin_server_address', t('Must be hostname or IP address, SSL connections usually require hostname not IP address'));
    }

    $c = $form_state['values']['uc_bitcoin_server_port'];
    if (!is_numeric($c) || $c < 0 || $c > 65535) {
      form_set_error('uc_bitcoin_server_port', t('Server port must be an integer and between 0 and 65535.'));
    }

    $c = $form_state['values']['uc_bitcoin_transaction_timeout'];
    if (!is_numeric($c) || $c <= 0) {
      form_set_error('uc_bitcoin_transaction_timeout', t('Transaction timeout must be greater than zero.'));
    }

    $c = $form_state['values']['uc_bitcoin_transaction_confirms'];
    if (!is_numeric($c) || $c < 0 || floatval($c) != intval($c)) {
      form_set_error('uc_bitcoin_transaction_confirms', t('Required transaction confirmations must be an integer greater than or equal to zero.'));
    }
  }

  // Check that we can connect
  if (!count(form_get_errors())) {
    switch ($form_state['values']['uc_bitcoin_conn_type']) {
      case 'Bitcoin daemon':
        // see if the connection works
        module_load_include('inc', 'uc_bitcoin', 'includes/bitcoin-php/bitcoin');
        $bitcoin_client = new BitcoinClient($form_state['values']['uc_bitcoin_server_scheme'],
          $form_state['values']['uc_bitcoin_server_username'],
          $form_state['values']['uc_bitcoin_server_password'],
          $form_state['values']['uc_bitcoin_server_address'],
          $form_state['values']['uc_bitcoin_server_port'],
          $form_state['values']['uc_bitcoin_server_certificate_path']);

        if (TRUE !== ($fault = $bitcoin_client->can_connect())) {
          form_set_error('', t("Unable to communicate with the Bitcoin server, check configuration or Bitcoin server status."));
        }
        break;
      case 'Blockchain.info':
        // check that composer_manage and Guzzle are alive
        if(!module_exists('composer_manager')) {
          form_set_error('', t('You need the composer_manager module installed - this module is required to load the modern Guzzle HTTP library.'));
        }
        if(!class_exists('Guzzle\Http\Client')) {
          form_set_error('', t('Guzzle HTTP library does not seem to be loading, is it installed by following composer_manager\'s steps?'));
          break;
        }

        // Do a balance check to verify the details
        $guid = $form_state['values']['uc_bitcoin_blockchain_guid'];
        $main_password = $form_state['values']['uc_bitcoin_blockchain_main_password'];

        $client = new Guzzle\Http\Client('https://blockchain.info');
        $request = $client->get("merchant/{$guid}/balance?password={$main_password}");
        $response = $request->send();

        $status = $response->getStatusCode();
        if ($status != 200) {
          form_set_error('', t('Some other network issue occured, please see the uc_bitcoin issue forum.'));
        }
        else {
          $result = json_decode($response->getBody());
          if (isset($result->error)) {
            form_set_error('uc_bitcoin_blockchain_guid', t('Could not get new_address, GUID is correct? Password or something else was wrong? (!error)', array('!error' => $result->error)));
          }
        }

        break;
    }
  }


}